/*
 * 
 * Copyright 2001-2004 The Ant-Contrib project
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package net.sf.antcontrib.cpptasks.sun;
import java.io.File;
import java.util.Vector;

import net.sf.antcontrib.cpptasks.CUtil;
import net.sf.antcontrib.cpptasks.compiler.LinkType;
import net.sf.antcontrib.cpptasks.compiler.Linker;
import net.sf.antcontrib.cpptasks.gcc.AbstractLdLinker;
/**
 * Adapter for Sun (r) Forte(tm) C Linker
 * 
 * @author Curt Arnold
 * @author Kenneth Russell
 */
public final class ForteCLinker extends ForteLinker {
    private static final ForteCLinker arLinker = new ForteCLinker(
            objFiles, discardFiles, "lib", ".a");
    private static final ForteCLinker dllLinker = new ForteCLinker(
            objFiles, discardFiles, "lib", ".so");
    private static final ForteCLinker instance = new ForteCLinker(
            objFiles, discardFiles, "", "");

    private ForteCLinker(String[] extensions,
            String[] ignoredExtensions, String outputPrefix, String outputSuffix) {
        super("cc", extensions, ignoredExtensions, outputPrefix, outputSuffix);
    }

    protected String getForteLinkerName() {
        return "cc";
    }

    public static ForteCLinker getInstance() {
        return instance;
    }

    public Linker getLinker(LinkType type) {
        if (type.isStaticLibrary()) {
            return arLinker;
        }
        if (type.isSharedLibrary()) {
            return dllLinker;
        }
        return instance;
    }
}
